from keeper.security import has_permission


class ObjectPermissionBackend(object):
    def authenticate(self, username, password):
        return None

    def has_perm(self, request, perm, obj, *args, **kwargs):
        return has_permission(perm, obj, request)



from django.template import Library
from keeper import security


register = Library()


@register.simple_tag(takes_context=True)
def has_permission(context, obj, permission):
    if obj is None:
        return False

    request = context['request']
    return security.has_permission(permission, obj, request)


@register.simple_tag(takes_context=True)
def has_global_permission(context, permission):
    request = context['request']
    return security.has_global_permission(permission, request)



from functools import wraps

from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.decorators import user_passes_test
from django.http import HttpResponseForbidden, Http404
from django.shortcuts import get_object_or_404, redirect as django_redirect

from .security import detect_permissions, GlobalContext


def login_required(login_url=None, redirect_field_name=REDIRECT_FIELD_NAME):
    # FIXME: Ugly hack to extract process when django's user_passes_test.
    return user_passes_test(
        lambda u: False,
        login_url=login_url,
        redirect_field_name=redirect_field_name,
    )(lambda r: "dummy")


def permission_denied(request, *args, **kwargs):
    return HttpResponseForbidden()


def not_found(request, *args, **kwargs):
    """ Hide page as 404
    * User accessed private contents but not permitted.
    """
    raise Http404


def redirect(name):
    """ Redirect to 'name' page
    * Redirect to "Buy Now" page
    """
    def _redirect(request, *args, **kwargs):
        return django_redirect(name)
    return _redirect


def keeper(permission,
           model=None, mapper=None,
           factory=None,
           on_fail=permission_denied):
    def dec(f):
        @wraps(f)
        def _wrapped(request, *args, **kwargs):
            if mapper:
                model_kwargs = mapper(request, *args, **kwargs)
                context = get_object_or_404(model, **model_kwargs)
            elif factory:
                context = factory(request, *args, **kwargs)
            else:
                context = GlobalContext()

            permissions = detect_permissions(context, request)

            request.k_context = context
            request.k_permissions = permissions

            if permission in permissions:
                return f(request, *args, **kwargs)
            else:
                return on_fail(request, *args, **kwargs)

        return _wrapped
    return dec



from importlib import import_module

from django.conf import settings


# Actions
Allow = 'allow'
Deny = 'deny'


def import_module_inner(path):
    try:
        module, func = path.rsplit('.', 1)
    except ImportError:
        return None
    return getattr(import_module(module), func, None)


GlobalContext = None


def initialize_global_context():
    if not hasattr(settings, 'KEEPER_GLOBAL_CONTEXT'):
        return

    global GlobalContext
    GlobalContext = import_module_inner(settings.KEEPER_GLOBAL_CONTEXT)


def detect_permissions(context, request):
    if not hasattr(context, '__acl__'):
        raise TypeError("Context %s doesn't have __acl__ attribute" % context)

    permissions = set()
    acl = context.__acl__()
    for action, operator, permission in acl:
        if isinstance(permission, tuple):
            p = set(permission)
        else:
            p = {permission}

        if isinstance(operator, type):
            o = operator()
        else:
            o = operator

        if o(request):
            if action is Allow:
                permissions |= p
            elif action is Deny:
                permissions -= p
    return permissions


def has_permission(permission, context, request):
    permissions = detect_permissions(context, request)
    return permission in permissions


def has_global_permission(permission, request):
    context = GlobalContext()
    return has_permission(permission, context, request)
